@@ -17,12 +17,15 @@ from codearkt.python_executor import PythonExecutor
 from codearkt.tools import fetch_tools
 from codearkt.event_bus import AgentEventBus, EventType
 from codearkt.llm import LLM, ChatMessages, ChatMessage
-from codearkt.util import get_unique_id
+from codearkt.util import get_unique_id, truncate_content
 
 DEFAULT_END_CODE_SEQUENCE = "<end_code>"
 DEFAULT_END_PLAN_SEQUENCE = "<end_plan>"
 DEFAULT_STOP_SEQUENCES = [DEFAULT_END_CODE_SEQUENCE, "Observation:", "Calling tools:"]
 AGENT_TOOL_PREFIX = "agent__"
+DEFAULT_MAX_ITERATIONS = 20
+PLANNING_LAST_N = 50
+PLANNING_CONTENT_MAX_LENGTH = 4000
 
 
 def extract_code_from_text(text: str) -> str | None:
@@ -79,7 +82,7 @@ class CodeActAgent:
         llm: LLM,
         tool_names: Sequence[str] = tuple(),
         prompts: Optional[Prompts] = None,
-        max_iterations: int = 10,
+        max_iterations: int = DEFAULT_MAX_ITERATIONS,
         verbosity_level: int = logging.ERROR,
         planning_interval: Optional[int] = None,
         managed_agents: Optional[List[Self]] = None,
@@ -281,6 +284,8 @@ class CodeActAgent:
         for stop_sequence in self.prompts.stop_sequences:
             if stop_sequence in output_text:
                 output_text = output_text.split(stop_sequence)[0].strip()
+                if stop_sequence == self.prompts.end_code_sequence:
+                    output_text += stop_sequence
                 break
 
         self._log(
@@ -347,7 +352,7 @@ class CodeActAgent:
         code_action = extract_code_from_text(content)
         assert code_action is None
         content = content.lower()
-        return "final" in content and "answer" in content
+        return "final answer:" in content
 
     async def _handle_final_message(
         self,
@@ -386,6 +391,40 @@ class CodeActAgent:
 
         return [ChatMessage(role="assistant", content=output_text)]
 
+    def _process_messages_for_planning(
+        self,
+        messages: ChatMessages,
+        last_n: int = PLANNING_LAST_N,
+        content_max_length: int = PLANNING_CONTENT_MAX_LENGTH,
+    ) -> str:
+        def messages_to_string(messages: ChatMessages) -> str:
+            str_messages = []
+            for m in messages:
+                content = truncate_content(str(m.content), max_length=content_max_length)
+                str_messages.append(f"{m.role}: {content}")
+            return "\n\n".join(str_messages)
+
+        assert self.prompts.plan_prefix is not None
+        assert self.prompts.plan_suffix is not None
+        plan_prefix = self.prompts.plan_prefix.render().strip()
+        plan_suffix = self.prompts.plan_suffix.render().strip()
+        used_messages = []
+        for message in messages:
+            if message.role == "system":
+                continue
+            content = str(message.content)
+            if plan_prefix in content or plan_suffix in content:
+                continue
+            used_messages.append(message)
+        if not used_messages:
+            return ""
+        if len(used_messages) <= last_n:
+            return messages_to_string(used_messages)
+        used_messages = used_messages[-last_n:]
+        conversation = messages_to_string(used_messages)
+        first_message = messages_to_string(used_messages[:1])
+        return f"First message:\n\n{first_message}\n\nLast {last_n} messages:\n\n{conversation}"
+
     async def _run_planning_step(
         self,
         messages: ChatMessages,
@@ -402,7 +441,7 @@ class CodeActAgent:
             self.prompts.plan_suffix is not None
         ), "Plan suffix is not set, but planning is enabled"
 
-        conversation = "\n\n".join([f"{m.role}: {m.content}" for m in messages[1:]])
+        conversation = self._process_messages_for_planning(messages)
         current_date = datetime.now().strftime("%Y-%m-%d")
         planning_prompt = self.prompts.plan.render(
             conversation=conversation, tools=tools, current_date=current_date
@@ -413,7 +452,7 @@ class CodeActAgent:
             output_stream = self.llm.astream(input_messages, stop=[self.prompts.end_plan_sequence])
 
             plan_prefix = self.prompts.plan_prefix.render().strip() + "\n\n"
-            await self._publish_event(event_bus, session_id, EventType.OUTPUT, plan_prefix)
+            await self._publish_event(event_bus, session_id, EventType.PLANNING_OUTPUT, plan_prefix)
             output_text = plan_prefix
 
             async for event in output_stream:
@@ -423,13 +462,19 @@ class CodeActAgent:
                     continue
                 assert isinstance(chunk, str), event
                 output_text += chunk
-                await self._publish_event(event_bus, session_id, EventType.OUTPUT, chunk)
+                await self._publish_event(event_bus, session_id, EventType.PLANNING_OUTPUT, chunk)
+                if self.prompts.end_plan_sequence in output_text:
+                    break
 
-            plan_suffix = "\n\n" + self.prompts.plan_suffix.render().strip() + "\n\n"
-            await self._publish_event(event_bus, session_id, EventType.OUTPUT, plan_suffix)
-            output_text += plan_suffix
+            if self.prompts.end_plan_sequence in output_text:
+                output_text = output_text.split(self.prompts.end_plan_sequence)[0].strip()
+                output_text += self.prompts.end_plan_sequence
 
-            return [ChatMessage(role="assistant", content=output_text)]
+            plan_suffix = self.prompts.plan_suffix.render().strip()
+            return [
+                ChatMessage(role="assistant", content=output_text),
+                ChatMessage(role="user", content=plan_suffix),
+            ]
 
         except Exception:
             exception = traceback.format_exc()
